<?php

using System::Text::Encoding;
using System::Text::RegularExpressions::Regex;
using Gtk;
using Gdk;

MainClass::Main();

class MainClass {

	// the media types we'll accept
	private static $target_table;

	// the media types we'll send
	private static $source_table;
		
	private static $monkey;
	
	static function Main() {
		Application::Init();

		self::$target_table = array(
			new TargetEntry("text/uri-list", TargetFlags::App, 0), 
			new TargetEntry("application/x-monkey", TargetFlags::App, 1)
		);
		
		self::$source_table = array(
			new TargetEntry("application/x-monkey", TargetFlags::App, 0)
		);	
		
		self::$monkey = new Pixbuf("monkey.png");
		
		$w = new Gtk::Window("Drag & drop");
		
		$h = new HBox();
		$h->BorderWidth = 6;
		$h->Spacing = 6;
		
		$image = new EventBox();
		$image->Add(new Gtk::Image(self::$monkey));
		$h->Add($image);

		$label = new Label("Drop stuff here");
		$h->Add($label);

		add_event($w, "DeleteEvent", new DeleteEventHandler(null, Window_Delete));
		$w->Add($h);
		$w->ShowAll();

		// set up label as a drop target
		Gtk::Drag::DestSet($label, DestDefaults::All, self::$target_table, DragAction::Copy);
		add_event($label, "DragDataReceived", new DragDataReceivedHandler(null, Data_Received));

		// set up image as a drag source
		Gtk::Drag::SourceSet($image, ModifierType::Button1Mask, self::$source_table, DragAction::Copy);
		add_event($image, "DragDataGet", new DragDataGetHandler(null, Data_Get));
		add_event($image, "DragBegin", new DragBeginHandler(null, Drag_Begin));
		Application::Run();
	}

	static function Window_Delete($o, DeleteEventArgs $args) {
		Application::Quit();
	}
	
	static function Data_Received($o, DragDataReceivedArgs $args) {
		$success = false;
		$source = Gtk::Drag::GetSourceWidget($args->Context);		
		$utf8 = Encoding::UTF8;
		$data = $utf8->GetString($args->SelectionData->Data);
		
		switch($args->Info) {
			case 0:	// uri-list
				$uri_list = Regex::Split($data, "\r\n");
				foreach ($uri_list as $u) {
					if ($u->Length > 0)
						echo "Got URI $u\n";
				}
				$success = true;
				break;
			case 1: // monkey
				echo "Monkey '$data' was dropped\n";
				$success = true;
				break;
		}
		
		Gtk::Drag::Finish($args->Context, $success, false, $args->Time);
	}
	
	static function Drag_Begin($o, DragBeginArgs $args) {
		Gtk::Drag::SetIconPixbuf($args->Context, self::$monkey, 0, 0);
	}

	static function Data_Get ($o, DragDataGetArgs $args) {
		$targets = $args->Context->Targets;
		$utf8 = Encoding::UTF8;
		$args->SelectionData->Set($targets[0], 8, $utf8->GetBytes("Rupert"));
	}
	
}

?>